@@ -1,101 +1,128 @@
+/** @file chm_common.c
+ */
 #include "chm_common.h"
 #include "Mutils.h"
 
 Rboolean isValid_Csparse(SEXP x); /* -> Csparse.c */
 
+SEXP get_SuiteSparse_version() {
+    SEXP ans = allocVector(INTSXP, 3);
+    int* version = INTEGER(ans);
+    SuiteSparse_version(version);
+    return ans;
+}
+
 cholmod_common c;
-/* cholmod_common cl; */
+cholmod_common cl;
 
 SEXP chm_common_env;
+static SEXP dboundSym, grow0Sym, grow1Sym, grow2Sym, maxrankSym,
+    supernodal_switchSym, supernodalSym, final_asisSym, final_superSym,
+    final_llSym, final_packSym, final_monotonicSym, final_resymbolSym,
+    prefer_zomplexSym, prefer_upperSym, quick_return_if_not_posdefSym,
+    nmethodsSym, m0_ordSym, postorderSym;
 
 void CHM_store_common() {
     SEXP rho = chm_common_env;
-    defineVar(install("dbound"), ScalarReal(c.dbound), rho);
-    defineVar(install("grow0"), ScalarReal(c.grow0), rho);
-    defineVar(install("grow1"), ScalarReal(c.grow1), rho);
-    defineVar(install("grow2"), ScalarInteger(c.grow2), rho);
-    defineVar(install("maxrank"), ScalarInteger(c.maxrank), rho);
-    defineVar(install("supernodal_switch"),
+    defineVar(dboundSym, ScalarReal(c.dbound), rho);
+    defineVar(grow0Sym, ScalarReal(c.grow0), rho);
+    defineVar(grow1Sym, ScalarReal(c.grow1), rho);
+    defineVar(grow2Sym, ScalarInteger(c.grow2), rho);
+    defineVar(maxrankSym, ScalarInteger(c.maxrank), rho);
+    defineVar(supernodal_switchSym,
 	      ScalarReal(c.supernodal_switch), rho);
-    defineVar(install("supernodal"),
-	      ScalarInteger(c.supernodal), rho);
-    defineVar(install("final_asis"),
-	      ScalarLogical(c.final_asis), rho);
-    defineVar(install("final_super"),
-	      ScalarLogical(c.final_super), rho);
-    defineVar(install("final_ll"),
-	      ScalarLogical(c.final_ll), rho);
-    defineVar(install("final_pack"),
-	      ScalarLogical(c.final_pack), rho);
-    defineVar(install("final_monotonic"),
-	      ScalarLogical(c.final_monotonic), rho);
-    defineVar(install("final_resymbol"),
-	      ScalarLogical(c.final_resymbol), rho);
-    defineVar(install("prefer_zomplex"),
-	      ScalarLogical(c.prefer_zomplex), rho);
-    defineVar(install("prefer_upper"),
-	      ScalarLogical(c.prefer_upper), rho);
-    defineVar(install("quick_return_if_not_posdef"),
+    defineVar(supernodalSym, ScalarInteger(c.supernodal), rho);
+    defineVar(final_asisSym, ScalarLogical(c.final_asis), rho);
+    defineVar(final_superSym, ScalarLogical(c.final_super), rho);
+    defineVar(final_llSym, ScalarLogical(c.final_ll), rho);
+    defineVar(final_packSym, ScalarLogical(c.final_pack), rho);
+    defineVar(final_monotonicSym, ScalarLogical(c.final_monotonic), rho);
+    defineVar(final_resymbolSym, ScalarLogical(c.final_resymbol), rho);
+    defineVar(prefer_zomplexSym, ScalarLogical(c.prefer_zomplex), rho);
+    defineVar(prefer_upperSym, ScalarLogical(c.prefer_upper), rho);
+    defineVar(quick_return_if_not_posdefSym,
 	      ScalarLogical(c.quick_return_if_not_posdef), rho);
-    defineVar(install("nmethods"), ScalarInteger(c.nmethods), rho);
-    defineVar(install("m0.ord"),
-	      ScalarInteger(c.method[0].ordering), rho);
-    defineVar(install("postorder"), ScalarLogical(c.postorder), rho);
+    defineVar(nmethodsSym, ScalarInteger(c.nmethods), rho);
+    defineVar(m0_ordSym, ScalarInteger(c.method[0].ordering), rho);
+    defineVar(postorderSym, ScalarLogical(c.postorder), rho);
 }
 
 void CHM_restore_common() {
     SEXP rho = chm_common_env;
-    c.dbound = asReal(findVarInFrame(rho, install("dbound")));
-    c.grow0 = asReal(findVarInFrame(rho, install("grow0")));
-    c.grow1 = asReal(findVarInFrame(rho, install("grow1")));
-    c.grow2 = asInteger(findVarInFrame(rho, install("grow2")));
-    c.maxrank =
-	asInteger(findVarInFrame(rho, install("maxrank")));
-    c.supernodal_switch =
-	asReal(findVarInFrame(rho, install("supernodal_switch")));
-    c.supernodal =
-	asLogical(findVarInFrame(rho, install("supernodal")));
-    c.final_asis =
-	asLogical(findVarInFrame(rho, install("final_asis")));
-    c.final_super =
-	asLogical(findVarInFrame(rho, install("final_super")));
-    c.final_ll =
-	asLogical(findVarInFrame(rho, install("final_ll")));
-    c.final_pack =
-	asLogical(findVarInFrame(rho, install("final_pack")));
-    c.final_monotonic =
-	asLogical(findVarInFrame(rho, install("final_monotonic")));
-    c.final_resymbol =
-	asLogical(findVarInFrame(rho, install("final_resymbol")));
-    c.prefer_zomplex =
-	asLogical(findVarInFrame(rho, install("prefer_zomplex")));
-    c.prefer_upper =
-	asLogical(findVarInFrame(rho, install("prefer_upper")));
+    c.dbound = asReal(findVarInFrame(rho, dboundSym));
+    c.grow0 = asReal(findVarInFrame(rho, grow0Sym));
+    c.grow1 = asReal(findVarInFrame(rho, grow1Sym));
+    c.grow2 = asInteger(findVarInFrame(rho, grow2Sym));
+    c.maxrank = asInteger(findVarInFrame(rho, maxrankSym));
+    c.supernodal_switch = asReal(findVarInFrame(rho, supernodal_switchSym));
+    c.supernodal = asLogical(findVarInFrame(rho, supernodalSym));
+    c.final_asis = asLogical(findVarInFrame(rho, final_asisSym));
+    c.final_super = asLogical(findVarInFrame(rho, final_superSym));
+    c.final_ll = asLogical(findVarInFrame(rho, final_llSym));
+    c.final_pack = asLogical(findVarInFrame(rho, final_packSym));
+    c.final_monotonic = asLogical(findVarInFrame(rho, final_monotonicSym));
+    c.final_resymbol = asLogical(findVarInFrame(rho, final_resymbolSym));
+    c.prefer_zomplex = asLogical(findVarInFrame(rho, prefer_zomplexSym));
+    c.prefer_upper = asLogical(findVarInFrame(rho, prefer_upperSym));
     c.quick_return_if_not_posdef =
-	asLogical(findVarInFrame(rho,
-				 install("quick_return_if_not_posdef")));
-    c.nmethods =
-	asInteger(findVarInFrame(rho, install("nmethods")));
-    c.method[0].ordering =
-	asInteger(findVarInFrame(rho, install("m0.ord")));
-    c.postorder =
-	asLogical(findVarInFrame(rho, install("postorder")));
+	asLogical(findVarInFrame(rho, quick_return_if_not_posdefSym));
+    c.nmethods = asInteger(findVarInFrame(rho, nmethodsSym));
+    c.method[0].ordering = asInteger(findVarInFrame(rho, m0_ordSym));
+    c.postorder = asLogical(findVarInFrame(rho, postorderSym));
 }
 
 SEXP CHM_set_common_env(SEXP rho) {
     if (!isEnvironment(rho))
 	error(_("Argument rho must be an environment"));
     chm_common_env = rho;
+    dboundSym = install("dbound");
+    grow0Sym = install("grow0");
+    grow1Sym = install("grow1");
+    grow2Sym = install("grow2");
+    maxrankSym = install("maxrank");
+    supernodal_switchSym = install("supernodal_switch");
+    supernodalSym = install("supernodal");
+    final_asisSym = install("final_asis");
+    final_superSym = install("final_super");
+    final_llSym = install("final_ll");
+    final_packSym = install("final_pack");
+    final_monotonicSym = install("final_monotonic");
+    final_resymbolSym = install("final_resymbol");
+    prefer_zomplexSym = install("final_zomplex");
+    prefer_upperSym = install("final_upper");
+    quick_return_if_not_posdefSym = install("quick_return_if_not_posdef");
+    nmethodsSym = install("nmethods");
+    m0_ordSym = install("m0.ord");
+    postorderSym = install("postorder");
     CHM_store_common();
     return R_NilValue;
 }
 
+/** @brief stype := "symmetry type".
+ *
+ *  ./CHOLMOD/Include/cholmod_core.h says about  'int stype' entry of cholmod_sparse_struct:
+ *    ------------------------------
+ * 0:  matrix is "unsymmetric": use both upper and lower triangular parts
+ *     (the matrix may actually be symmetric in pattern and value, but
+ *     both parts are explicitly stored and used).  May be square or
+ *     rectangular.
+ * >0: matrix is square and symmetric, use upper triangular part.
+ *     Entries in the lower triangular part are ignored.
+ * <0: matrix is square and symmetric, use lower triangular part.
+ *     Entries in the upper triangular part are ignored.
+ */
 static int stype(int ctype, SEXP x)
 {
     if ((ctype % 3) == 1) return (*uplo_P(x) == 'U') ? 1 : -1;
     return 0;
 }
 
+/** @brief xtype: the _kind_ of numeric (think "x slot") of Cholmod sparse matrices.
+  #define CHOLMOD_PATTERN 0	 pattern only, no numerical values
+  #define CHOLMOD_REAL    1	 a real matrix
+  #define CHOLMOD_COMPLEX 2	 a complex matrix (ANSI C99 compatible)
+  #define CHOLMOD_ZOMPLEX 3	 a complex matrix (MATLAB compatible)
+*/
 static int xtype(int ctype)
 {
     switch(ctype / 3) {
@@ -165,13 +192,13 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
 
     /* R_alloc the vector storage for dest and copy the contents from src */
     np1 = src->ncol + 1;
-    nnz = (int) cholmod_l_nnz(src, &c);
-    dest->p = (void*) Memcpy((int*)R_alloc(sizeof(int), np1),
+    nnz = (int) cholmod_nnz(src, &c);
+    dest->p = (void*) Memcpy((int*)R_alloc(np1, sizeof(int)),
 			     (int*)(src->p), np1);
-    dest->i = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+    dest->i = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
 			     (int*)(src->i), nnz);
     if(src->xtype)
-	dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
+	dest->x = (void*) Memcpy((double*)R_alloc(nnz, sizeof(double)),
 				 (double*)(src->x), nnz);
 }
 
@@ -187,12 +214,12 @@ static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
 
     /* R_alloc the vector storage for dest and copy the contents from src */
     nnz = src->nnz;
-    dest->i = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+    dest->i = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
 			     (int*)(src->i), nnz);
-    dest->j = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+    dest->j = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
 			     (int*)(src->j), nnz);
     if(src->xtype)
-	dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
+	dest->x = (void*) Memcpy((double*)R_alloc(nnz, sizeof(double)),
 				 (double*)(src->x), nnz);
 }
 
@@ -218,11 +245,7 @@ static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
 CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 			 Rboolean check_Udiag, Rboolean sort_in_place)
 {
-    char *valid[] = {"dgCMatrix", "dsCMatrix", "dtCMatrix",
-		     "lgCMatrix", "lsCMatrix", "ltCMatrix",
-		     "ngCMatrix", "nsCMatrix", "ntCMatrix",
-		     "zgCMatrix", "zsCMatrix", "ztCMatrix",
-		     ""};
+    static const char *valid[] = { MATRIX_VALID_Csparse, ""};
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
 	ctype = Matrix_check_class_etc(x, valid);
     SEXP islot = GET_SLOT(x, Matrix_iSym);
@@ -232,7 +255,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 	error(_("invalid object passed to as_cholmod_sparse"));
     memset(ans, 0, sizeof(cholmod_sparse)); /* zero the struct */
 
-    ans->itype = CHOLMOD_LONG;	/* characteristics of the system */
+    ans->itype = CHOLMOD_INT;	/* characteristics of the system */
     ans->dtype = CHOLMOD_DOUBLE;
     ans->packed = TRUE;
 				/* slots always present */
@@ -255,36 +278,37 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
     ans->sorted = check_sorted_chm(ans);
     if (!(ans->sorted)) { /* sort columns */
 	if(sort_in_place) {
-	    if (!cholmod_l_sort(ans, &c))
-		error(_("in_place cholmod_l_sort returned an error code"));
+	    if (!cholmod_sort(ans, &c))
+		error(_("in_place cholmod_sort returned an error code"));
 	    ans->sorted = 1;
 	}
 	else {
-	    CHM_SP tmp = cholmod_l_copy_sparse(ans, &c);
-	    if (!cholmod_l_sort(tmp, &c))
-		error(_("cholmod_l_sort returned an error code"));
+	    CHM_SP tmp = cholmod_copy_sparse(ans, &c);
+	    if (!cholmod_sort(tmp, &c))
+		error(_("cholmod_sort returned an error code"));
 
 #ifdef DEBUG_Matrix
 	    /* This "triggers" exactly for return values of dtCMatrix_sparse_solve():*/
 	    /* Don't want to translate this: want it report */
-	    Rprintf("Note: as_cholmod_sparse() needed cholmod_l_sort()ing\n");
+	    Rprintf("Note: as_cholmod_sparse() needed cholmod_sort()ing\n");
 #endif
 	    chm2Ralloc(ans, tmp);
-	    cholmod_l_free_sparse(&tmp, &c);
+	    cholmod_free_sparse(&tmp, &c);
 	}
     }
 
-    if (check_Udiag && ctype % 3 == 2 && (*diag_P(x) == 'U')) { /* diagU2N(.)  "in place" : */
+    if (check_Udiag && ctype % 3 == 2 // triangular
+	&& (*diag_P(x) == 'U')) { /* diagU2N(.)  "in place" : */
 	double one[] = {1, 0};
-	CHM_SP eye = cholmod_l_speye(ans->nrow, ans->ncol, ans->xtype, &c);
-	CHM_SP tmp = cholmod_l_add(ans, eye, one, one, TRUE, TRUE, &c);
+	CHM_SP eye = cholmod_speye(ans->nrow, ans->ncol, ans->xtype, &c);
+	CHM_SP tmp = cholmod_add(ans, eye, one, one, TRUE, TRUE, &c);
 
-#ifdef DEBUG_Matrix_verbose /* quite often, e.g. in ../tests/indexing.R */
+#ifdef DEBUG_Matrix_verbose /* happens quite often, e.g. in ../tests/indexing.R : */
 	Rprintf("Note: as_cholmod_sparse(<ctype=%d>) - diagU2N\n", ctype);
 #endif
 	chm2Ralloc(ans, tmp);
-	cholmod_l_free_sparse(&tmp, &c);
-	cholmod_l_free_sparse(&eye, &c);
+	cholmod_free_sparse(&tmp, &c);
+	cholmod_free_sparse(&eye, &c);
     } /* else :
        * NOTE: if(*diag_P(x) == 'U'), the diagonal is lost (!);
        * ---- that may be ok, e.g. if we are just converting from/to Tsparse,
@@ -298,10 +322,10 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
  * optionally, free a or free both a and its the pointers to its contents.
  *
  * @param a  (cholmod_sparse) matrix to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_l_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
  * @param uploT 0 - not triangular; > 0 upper triangular; < 0 lower
  * @param Rkind - vector type to store for a->xtype == CHOLMOD_REAL,
- *                0 - REAL; 1 - LOGICAL
+ *                0 - REAL; 1 - LOGICAL  [unused for other a->xtype]
  * @param diag character string suitable for the diag slot of a
  *          triangular matrix (not accessed if uploT == 0).
  * @param dn either R_NilValue or an SEXP suitable for the Dimnames slot.
@@ -313,13 +337,22 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 {
     SEXP ans;
     char *cls = "";/* -Wall */
-    int *dims, nnz, *aii = (int*)(a->i), *api = (int*)(a->p);
+    int *dims, nnz, *ansp, *ansi, *aii = (int*)(a->i), *api = (int*)(a->p),
+	longi = (a->itype) == CHOLMOD_LONG;
+    SuiteSparse_long *ail = (SuiteSparse_long*)(a->i), *apl = (SuiteSparse_long*)(a->p);
 
     PROTECT(dn);  /* dn is usually UNPROTECTed before the call */
 
 				/* ensure a is sorted and packed */
-    if (!a->sorted || !a->packed) cholmod_l_sort(a, &c);
+    if (!a->sorted || !a->packed)
+	longi ? cholmod_l_sort(a, &cl) : cholmod_sort(a, &c);
 				/* determine the class of the result */
+
+#define DOFREE_MAYBE							\
+    if (dofree > 0)							\
+	longi ? cholmod_l_free_sparse(&a, &cl) : cholmod_free_sparse(&a, &c); \
+    else if (dofree < 0) Free(a)
+
     switch(a->xtype){
     case CHOLMOD_PATTERN:
 	cls = uploT ? "ntCMatrix": ((a->stype) ? "nsCMatrix" : "ngCMatrix");
@@ -333,21 +366,26 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 	    cls = uploT ? "ltCMatrix": ((a->stype) ? "lsCMatrix" : "lgCMatrix");
 	    break;
 	default:
+	    DOFREE_MAYBE;
 	    error(_("chm_sparse_to_SEXP(<real>, *): invalid 'Rkind' (real kind code)"));
 	}
 	break;
     case CHOLMOD_COMPLEX:
 	cls = uploT ? "ztCMatrix": ((a->stype) ? "zsCMatrix" : "zgCMatrix");
 	break;
-    default: error(_("unknown xtype in cholmod_sparse object"));
+    default:
+	DOFREE_MAYBE;
+	error(_("unknown xtype in cholmod_sparse object"));
     }
     ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cls)));
 				/* allocate and copy common slots */
-    nnz = cholmod_l_nnz(a, &c);
+    nnz = longi ? cholmod_l_nnz(a, &cl) : cholmod_nnz(a, &c);
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = a->nrow; dims[1] = a->ncol;
-    Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, a->ncol + 1)), api, a->ncol + 1);
-    Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_iSym, INTSXP, nnz)), aii, nnz);
+    ansp = INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, a->ncol + 1));
+    ansi = INTEGER(ALLOC_SLOT(ans, Matrix_iSym, INTSXP, nnz));
+    for (int j = 0; j <= a->ncol; j++) ansp[j] = longi ? (int)(apl[j]) : api[j];
+    for (int p = 0; p < nnz; p++) ansi[p] = longi ? (int)(ail[p]) : aii[p];
 				/* copy data slot if present */
     if (a->xtype == CHOLMOD_REAL) {
 	int i, *m_x;
@@ -360,14 +398,16 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 	case 1:
 	    m_x = LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, nnz));
 	    for (i=0; i < nnz; i++)
-		m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (int) a_x[i]; /* or f != 0 */
+		m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (a_x[i] != 0);
 	    break;
 	}
     }
-    else if (a->xtype == CHOLMOD_COMPLEX)
+    else if (a->xtype == CHOLMOD_COMPLEX) {
+	DOFREE_MAYBE;
 	error(_("complex sparse matrix code not yet written"));
 /* 	Memcpy(COMPLEX(ALLOC_SLOT(ans, Matrix_xSym, CPLXSXP, nnz)), */
 /* 	       (complex *) a->x, nnz); */
+    }
     if (uploT) {		/* slots for triangularMatrix */
 	if (a->stype) error(_("Symmetric and triangular both set"));
 	SET_SLOT(ans, Matrix_uploSym, mkString((uploT > 0) ? "U" : "L"));
@@ -376,14 +416,50 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
     if (a->stype)		/* slot for symmetricMatrix */
 	SET_SLOT(ans, Matrix_uploSym,
 		 mkString((a->stype > 0) ? "U" : "L"));
-    if (dofree > 0) cholmod_l_free_sparse(&a, &c);
-    if (dofree < 0) Free(a);
+    DOFREE_MAYBE;
     if (dn != R_NilValue)
 	SET_SLOT(ans, Matrix_DimNamesSym, duplicate(dn));
 
     UNPROTECT(2);
     return ans;
 }
+#undef DOFREE_MAYBE
+
+
+/**
+* Change the "type" of a cholmod_sparse matrix, i.e. modify it "in place"
+*
+* @param to_xtype requested xtype (pattern, real, complex, zomplex)
+* @param A sparse matrix to change
+* @param Common cholmod's common
+*
+* @return TRUE/FALSE , TRUE iff success
+*/
+Rboolean chm_MOD_xtype(int to_xtype, cholmod_sparse *A, CHM_CM Common) {
+//     *MOD*: shouting, as A is modified in place
+
+/* --------------------------------------------------------------------------
+ * cholmod_sparse_xtype: change the xtype of a sparse matrix
+ * --------------------------------------------------------------------------
+  int cholmod_sparse_xtype
+  (
+      // ---- input ----
+      int to_xtype,	//
+      // ---- in/out ---
+      cholmod_sparse *A, //
+      // ---------------
+      cholmod_common *Common
+  ) ;
+
+  int cholmod_l_sparse_xtype (int, cholmod_sparse *, cholmod_common *) ;
+*/
+    if((A->itype) == CHOLMOD_LONG) {
+	return (Rboolean) cholmod_l_sparse_xtype (to_xtype, A, Common);
+    } else {
+	return (Rboolean) cholmod_sparse_xtype   (to_xtype, A, Common);
+    }
+}
+
 
 /**
  * Populate ans with the pointers from x and modify its scalar
@@ -403,13 +479,9 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
  */
 CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 {
-    char *valid[] = {"dgTMatrix", "dsTMatrix", "dtTMatrix",
-		     "lgTMatrix", "lsTMatrix", "ltTMatrix",
-		     "ngTMatrix", "nsTMatrix", "ntTMatrix",
-		     "zgTMatrix", "zsTMatrix", "ztTMatrix",
-		     ""};
-    int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	ctype = Matrix_check_class_etc(x, valid);
+    static const char *valid[] = { MATRIX_VALID_Tsparse, ""};
+    int ctype = Matrix_check_class_etc(x, valid),
+	*dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
     SEXP islot = GET_SLOT(x, Matrix_iSym);
     int m = LENGTH(islot);
     Rboolean do_Udiag = (check_Udiag && ctype % 3 == 2 && (*diag_P(x) == 'U'));
@@ -417,7 +489,7 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 
     memset(ans, 0, sizeof(cholmod_triplet)); /* zero the struct */
 
-    ans->itype = CHOLMOD_LONG;	/* characteristics of the system */
+    ans->itype = CHOLMOD_INT;	/* characteristics of the system */
     ans->dtype = CHOLMOD_DOUBLE;
 				/* nzmax, dimensions, types and slots : */
     ans->nnz = ans->nzmax = m;
@@ -436,7 +508,7 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 	CHM_TR tmp = cholmod_l_copy_triplet(ans, &c);
 	int *a_i, *a_j;
 
-	if(!cholmod_l_reallocate_triplet((size_t) k, tmp, &c))
+	if(!cholmod_reallocate_triplet((size_t) k, tmp, &c))
 	    error(_("as_cholmod_triplet(): could not reallocate for internal diagU2N()"
 		      ));
 
@@ -488,7 +560,7 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
  * optionally, free a or free both a and its the pointers to its contents.
  *
  * @param a matrix to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_l_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
  * @param uploT 0 - not triangular; > 0 upper triangular; < 0 lower
  * @param Rkind - vector type to store for a->xtype == CHOLMOD_REAL,
  *                0 - REAL; 1 - LOGICAL
@@ -507,6 +579,11 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 
     PROTECT(dn);  /* dn is usually UNPROTECTed before the call */
 				/* determine the class of the result */
+
+#define DOFREE_MAYBE					\
+    if (dofree > 0) cholmod_free_triplet(&a, &c);	\
+    else if (dofree < 0) Free(a)
+
     switch(a->xtype) {
     case CHOLMOD_PATTERN:
 	cl = uploT ? "ntTMatrix" :
@@ -528,7 +605,9 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 	cl = uploT ? "ztTMatrix" :
 	    ((a->stype) ? "zsTMatrix" : "zgTMatrix");
 	break;
-    default: error(_("unknown xtype in cholmod_triplet object"));
+    default:
+	DOFREE_MAYBE;
+	error(_("unknown xtype in cholmod_triplet object"));
     }
     ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
 				/* allocate and copy common slots */
@@ -548,16 +627,18 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 		   a_x, a->nnz);
 	    break;
 	case 1:
-	    m_x= LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, a->nnz));
+	    m_x = LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, a->nnz));
 	    for (i=0; i < a->nnz; i++)
-		m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (int) a_x[i]; /* or f != 0 */
+		m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (a_x[i] != 0);
 	    break;
 	}
     }
-    else if (a->xtype == CHOLMOD_COMPLEX)
+    else if (a->xtype == CHOLMOD_COMPLEX) {
+	DOFREE_MAYBE;
 	error(_("complex sparse matrix code not yet written"));
 /* 	Memcpy(COMPLEX(ALLOC_SLOT(ans, Matrix_xSym, CPLXSXP, a->nnz)), */
 /* 	       (complex *) a->x, a->nz); */
+    }
     if (uploT) {		/* slots for triangularMatrix */
 	if (a->stype) error(_("Symmetric and triangular both set"));
 	SET_SLOT(ans, Matrix_uploSym, mkString((uploT > 0) ? "U" : "L"));
@@ -567,13 +648,13 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
     if (a->stype)
 	SET_SLOT(ans, Matrix_uploSym,
 		 mkString((a->stype > 0) ? "U" : "L"));
-    if (dofree > 0) cholmod_l_free_triplet(&a, &c);
-    if (dofree < 0) Free(a);
+    DOFREE_MAYBE;
     if (dn != R_NilValue)
 	SET_SLOT(ans, Matrix_DimNamesSym, duplicate(dn));
     UNPROTECT(2);
     return ans;
 }
+#undef DOFREE_MAYBE
 
 /**
  * Populate ans with the pointers from x and modify its scalar
@@ -591,11 +672,8 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 CHM_DN as_cholmod_dense(CHM_DN ans, SEXP x)
 {
 #define _AS_cholmod_dense_1						\
-    char *valid[] = {"dmatrix", "dgeMatrix",				\
-		     "lmatrix", "lgeMatrix",				\
-		     "nmatrix", "ngeMatrix",				\
-		     "zmatrix", "zgeMatrix", ""};			\
-    int dims[2], ctype = Matrix_check_class_etc(x, valid), nprot = 0; \
+    static const char *valid[] = { MATRIX_VALID_ge_dense, ""};		\
+    int dims[2], ctype = Matrix_check_class_etc(x, valid), nprot = 0;	\
 									\
     if (ctype < 0) {		/* not a classed matrix */		\
 	if (isMatrix(x)) Memcpy(dims, INTEGER(getAttrib(x, R_DimSymbol)), 2); \
@@ -650,7 +728,7 @@ CHM_DN as_cholmod_dense(CHM_DN ans, SEXP x)
 /* version of as_cholmod_dense() that produces a cholmod_dense matrix
  * with REAL 'x' slot -- i.e. treats "nMatrix" as "lMatrix" -- as only difference;
  * Not just via a flag in as_cholmod_dense() since that has fixed API */
-CHM_DN as_cholmod_x_dense(cholmod_dense *ans, SEXP x)
+CHM_DN as_cholmod_x_dense(CHM_DN ans, SEXP x)
 {
     _AS_cholmod_dense_1;
 
@@ -666,6 +744,28 @@ CHM_DN as_cholmod_x_dense(cholmod_dense *ans, SEXP x)
 #undef _AS_cholmod_dense_1
 #undef _AS_cholmod_dense_2
 
+/**
+* Transpose a cholmod_dense matrix  ("too trivial to be in CHOLMOD?")
+*
+* @param ans (pointer to) already allocated result of correct dimension
+* @param x   (pointer to) cholmod_dense matrix to be transposed
+*
+*/
+void chm_transpose_dense(CHM_DN ans, CHM_DN x)
+{
+    if (x->xtype != CHOLMOD_REAL)
+	error(_("chm_transpose_dense(ans, x) not yet implemented for %s different from %s"),
+	      "x->xtype", "CHOLMOD_REAL");
+    double *xx = x->x, *ansx = ans->x;
+    // Inspired from R's do_transpose() in .../R/src/main/array.c :
+    int i,j, nrow = x->nrow, len = x->nzmax, l_1 = len-1;
+    for (i = 0, j = 0; i < len; i++, j += nrow) {
+	if (j > l_1) j -= l_1;
+	ansx[i] = xx[j];
+    }
+    return;
+}
+
 void R_cholmod_error(int status, const char *file, int line, const char *message)
 {
     CHM_restore_common(); /* restore any setting that may have been changed */
@@ -678,11 +778,11 @@ void R_cholmod_error(int status, const char *file, int line, const char *message
 #ifndef R_CHOLMOD_ALWAYS_ERROR
     if(status < 0) {
 #endif
-	error(_("Cholmod error '%s' at file:%s, line %d"), message, file, line);
+	error(_("Cholmod error '%s' at file %s, line %d"), message, file, line);
 #ifndef R_CHOLMOD_ALWAYS_ERROR
     }
     else
-	warning(_("Cholmod warning '%s' at file:%s, line %d"),
+	warning(_("Cholmod warning '%s' at file %s, line %d"),
 		message, file, line);
 #endif
 }
@@ -703,23 +803,20 @@ int R_cholmod_printf(const char* fmt, ...)
  * Initialize the CHOLMOD library and replace the print and error functions
  * by R-specific versions.
  *
- * @param cl pointer to a cholmod_common structure to be initialized
+ * @param c pointer to a cholmod_common structure to be initialized
  *
  * @return TRUE if successful
  */
-int R_cholmod_l_start(CHM_CM cl)
+int R_cholmod_start(CHM_CM c)
 {
-/* NB: keep in sync with M_R_cholmod_l_start(), ../inst/include/Matrix_stubs.c */
     int res;
-    if (!(res = cholmod_l_start(cl)))
-	error(_("Unable to initialize cholmod_l: error code %d"), res);
-    cl->print_function = R_cholmod_printf; /* Rprintf gives warning */
-    /* Since we provide an error handler, it may not be a good idea to
-     * allow CHOLMOD printing.
-     * FIXME? alternative {slightly more efficient}:
-     * do *compile* CHOLMOD with '#define NPRINT'  */
-    cl->print_function = NULL;
-    cl->error_handler = R_cholmod_error;
+    if (!(res = cholmod_start(c)))
+	error(_("Unable to initialize cholmod: error code %d"), res);
+    c->print_function = R_cholmod_printf; /* Rprintf gives warning */
+    /* Since we provide an error handler, it may not be a good idea to allow CHOLMOD printing,
+     * because that's not easily suppressed on the R level :
+     * Hence consider, at least temporarily *  c->print_function = NULL; */
+    c->error_handler = R_cholmod_error;
     return TRUE;
 }
 
@@ -728,22 +825,25 @@ int R_cholmod_l_start(CHM_CM cl)
  * optionally, free a or free both a and its pointer to its contents.
  *
  * @param a matrix to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_l_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
  * @param Rkind type of R matrix to be generated (special to this function)
- * @param dn   -- dimnames [list(.,.) or NULL]
+ * @param dn   -- dimnames [list(.,.) or NULL;  __already__ transposed when transp is TRUE ]
+ * @param transp Rboolean, if TRUE, the result must be a copy of  t(a), i.e., "a transposed"
  *
  * @return SEXP containing a copy of a
  */
-
-/* FIXME: should also have args  (int uploST, char *diag) */
-
-SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn)
+SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn, Rboolean transp)
 {
+/* FIXME: should also have args  (int uploST, char *diag) */
     SEXP ans;
     char *cl = ""; /* -Wall */
     int *dims, ntot;
 
-    PROTECT(dn); /* << (why? -- just cut&paste from chm_dense_to_mat.. below*/
+    PROTECT(dn); // <-- no longer protected in caller
+
+#define DOFREE_de_MAYBE				\
+    if (dofree > 0) cholmod_free_dense(&a, &c);	\
+    else if (dofree < 0) Free(a);
 
     switch(a->xtype) {		/* determine the class of the result */
 /* CHOLMOD_PATTERN never happens because cholmod_dense can't :
@@ -755,46 +855,74 @@ SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn)
 	case -1: cl = "ngeMatrix"; break;
 	case 0:	 cl = "dgeMatrix"; break;
 	case 1:	 cl = "lgeMatrix"; break;
-	default: error(_("unknown 'Rkind'"));
+	default:
+	    DOFREE_de_MAYBE;
+	    error(_("unknown 'Rkind'"));
 	}
 	break;
     case CHOLMOD_COMPLEX:
 	cl = "zgeMatrix"; break;
     default:
+	DOFREE_de_MAYBE;
 	error(_("unknown xtype"));
     }
 
     ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
 				/* allocate and copy common slots */
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
-    dims[0] = a->nrow; dims[1] = a->ncol;
+    if(transp) {
+	dims[1] = a->nrow; dims[0] = a->ncol;
+    } else {
+	dims[0] = a->nrow; dims[1] = a->ncol;
+    }
     ntot = dims[0] * dims[1];
-    if (a->d == a->nrow) {	/* copy data slot -- always present in dense(!) */
+    if (a->d == a->nrow) { /* copy data slot -- always present in dense(!) */
 	if (a->xtype == CHOLMOD_REAL) {
 	    int i, *m_x;
+	    double *ansx, *a_x = (double *) a->x;
 	    switch(Rkind) {
 	    case 0:
-		Memcpy(REAL(ALLOC_SLOT(ans, Matrix_xSym, REALSXP, ntot)),
-		       (double *) a->x, ntot);
+		ansx = REAL(ALLOC_SLOT(ans, Matrix_xSym, REALSXP, ntot));
+		if(transp) {
+		    // Inspired from R's do_transpose() in .../R/src/main/array.c :
+		    int i,j, nrow = a->nrow, len = ntot, l_1 = len-1;
+		    for (i = 0, j = 0; i < len; i++, j += nrow) {
+			if (j > l_1) j -= l_1;
+			ansx[i] = a_x[j];
+		    }
+		} else {
+		    Memcpy(ansx, a_x, ntot);
+		}
 		break;
 	    case -1: /* nge*/
 	    case 1:  /* lge*/
 		m_x = LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, ntot));
-		for (i=0; i < ntot; i++)
-		    m_x[i] = (int) ((double *) a->x)[i];
-/* 		Memcpy(LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, ntot)), */
-/* 		       (int *) a->x, ntot); */
+		if(transp) {
+		    // Inspired from R's do_transpose() in .../R/src/main/array.c :
+		    int i,j, nrow = a->nrow, len = ntot, l_1 = len-1;
+		    for (i = 0, j = 0; i < len; i++, j += nrow) {
+			if (j > l_1) j -= l_1;
+			m_x[i] = a_x[j];
+		    }
+		} else {
+		    for (i=0; i < ntot; i++)
+			m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (a_x[i] != 0);
+		}
 		break;
 	    }
 	}
-	else if (a->xtype == CHOLMOD_COMPLEX)
+	else if (a->xtype == CHOLMOD_COMPLEX) {
+	    DOFREE_de_MAYBE;
 	    error(_("complex sparse matrix code not yet written"));
 /*	Memcpy(COMPLEX(ALLOC_SLOT(ans, Matrix_xSym, CPLXSXP, ntot)), */
 /*	       (complex *) a->x, ntot); */
-    } else error(_("code for cholmod_dense with holes not yet written"));
+	}
+    } else {
+	DOFREE_de_MAYBE;
+	error(_("code for cholmod_dense with holes not yet written"));
+    }
 
-    if (dofree > 0) cholmod_l_free_dense(&a, &c);
-    if (dofree < 0) Free(a);
+    DOFREE_de_MAYBE;
     if (dn != R_NilValue)
 	SET_SLOT(ans, Matrix_DimNamesSym, duplicate(dn));
     UNPROTECT(2);
@@ -805,44 +933,77 @@ SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn)
  * Copy the contents of a to a matrix object and, optionally, free a
  * or free both a and its pointer to its contents.
  *
- * @param a cholmod_dense structure to be converted
- * @param dofree 0 - don't free a; > 0 cholmod_l_free a; < 0 Free a
+ * @param a cholmod_dense structure to be converted {already REAL for original l..CMatrix}
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
  * @param dn either R_NilValue or an SEXP suitable for the Dimnames slot.
  *
  * @return SEXP containing a copy of a as a matrix object
  */
 SEXP chm_dense_to_matrix(CHM_DN a, int dofree, SEXP dn)
 {
-    SEXP ans;
-    SEXPTYPE typ;
+#define CHM_DENSE_TYPE						\
+    SEXPTYPE typ;						\
+    /* determine the class of the result */			\
+    typ = (a->xtype == CHOLMOD_PATTERN) ? LGLSXP :		\
+	((a->xtype == CHOLMOD_REAL) ? REALSXP :			\
+	 ((a->xtype == CHOLMOD_COMPLEX) ? CPLXSXP : NILSXP));	\
+    if (typ == NILSXP) {					\
+	DOFREE_de_MAYBE;					\
+	error(_("unknown xtype"));				\
+    }
 
     PROTECT(dn);
-				/* determine the class of the result */
-    typ = (a->xtype == CHOLMOD_PATTERN) ? LGLSXP :
-	((a->xtype == CHOLMOD_REAL) ? REALSXP :
-	 ((a->xtype == CHOLMOD_COMPLEX) ? CPLXSXP : NILSXP));
-    if (typ == NILSXP) error(_("unknown xtype"));
+    CHM_DENSE_TYPE;
 
-    ans = PROTECT(allocMatrix(typ, a->nrow, a->ncol));
-    if (a->d == a->nrow) {	/* copy data slot if present */
-	if (a->xtype == CHOLMOD_REAL)
-	    Memcpy(REAL(ans), (double *) a->x, a->nrow * a->ncol);
-	else if (a->xtype == CHOLMOD_COMPLEX)
-	    error(_("complex sparse matrix code not yet written"));
-	else if (a->xtype == CHOLMOD_PATTERN)
-	    error(_("don't know if a dense pattern matrix makes sense"));
-/* 	Memcpy(COMPLEX(ALLOC_SLOT(ans, Matrix_xSym, CPLXSXP, a->nnz)), */
-/* 	       (complex *) a->x, a->nz); */
-    } else error(_("code for cholmod_dense with holes not yet written"));
+    SEXP ans = PROTECT(allocMatrix(typ, a->nrow, a->ncol));
 
-    if (dofree > 0) cholmod_l_free_dense(&a, &c);
-    if (dofree < 0) Free(a);
+#define CHM_DENSE_COPY_DATA						\
+    if (a->d == a->nrow) {	/* copy data slot if present */		\
+	if (a->xtype == CHOLMOD_REAL)					\
+	    Memcpy(REAL(ans), (double *) a->x, a->nrow * a->ncol);	\
+	else if (a->xtype == CHOLMOD_COMPLEX) {				\
+	    DOFREE_de_MAYBE;						\
+	    error(_("complex sparse matrix code not yet written"));	\
+/* 	Memcpy(COMPLEX(ALLOC_SLOT(ans, Matrix_xSym, CPLXSXP, a->nnz)), */ \
+/* 	       (complex *) a->x, a->nz); */				\
+	} else if (a->xtype == CHOLMOD_PATTERN) {			\
+	    DOFREE_de_MAYBE;						\
+	    error(_("don't know if a dense pattern matrix makes sense")); \
+	}								\
+    } else {								\
+	DOFREE_de_MAYBE;						\
+	error(_("code for cholmod_dense with holes not yet written"));	\
+    }
+
+    CHM_DENSE_COPY_DATA;
+
+    DOFREE_de_MAYBE;
     if (dn != R_NilValue)
         setAttrib(ans, R_DimNamesSymbol, duplicate(dn));
     UNPROTECT(2);
     return ans;
 }
 
+/**
+ * Copy the contents of a to a numeric R object and, optionally, free a
+ * or free both a and its pointer to its contents.
+ *
+ * @param a cholmod_dense structure to be converted
+ * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
+ *
+ * @return SEXP containing a copy of a  in the sense of  as.vector(a)
+ */
+SEXP chm_dense_to_vector(CHM_DN a, int dofree)
+{
+    CHM_DENSE_TYPE;
+
+    SEXP ans = PROTECT(allocVector(typ, a->nrow * a->ncol));
+    CHM_DENSE_COPY_DATA;
+    DOFREE_de_MAYBE;
+    UNPROTECT(1);
+    return ans;
+}
+
 CHM_DN numeric_as_chm_dense(CHM_DN ans, double *v, int nr, int nc)
 {
     ans->d = ans->nrow = nr;
@@ -869,7 +1030,7 @@ CHM_DN numeric_as_chm_dense(CHM_DN ans, double *v, int nr, int nc)
  */
 CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x)
 {
-    char *valid[] = {"dCHMsuper", "dCHMsimpl", "nCHMsuper", "nCHMsimpl", ""};
+    static const char *valid[] = { MATRIX_VALID_CHMfactor, ""};
     int *type = INTEGER(GET_SLOT(x, install("type"))),
 	ctype = Matrix_check_class_etc(x, valid);
     SEXP tmp;
@@ -877,7 +1038,7 @@ CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x)
     if (ctype < 0) error(_("invalid class of object to as_cholmod_factor"));
     memset(ans, 0, sizeof(cholmod_factor)); /* zero the struct */
 
-    ans->itype = CHOLMOD_LONG;	/* characteristics of the system */
+    ans->itype = CHOLMOD_INT;	/* characteristics of the system */
     ans->dtype = CHOLMOD_DOUBLE;
     ans->z = (void *) NULL;
     ans->xtype = (ctype < 2) ? CHOLMOD_REAL : CHOLMOD_PATTERN;
@@ -927,11 +1088,12 @@ CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x)
 	ans->next = INTEGER(GET_SLOT(x, install("nxt")));
 	ans->prev = INTEGER(GET_SLOT(x, install("prv")));
     }
-    if (!cholmod_l_check_factor(ans, &c))
+    if (!cholmod_check_factor(ans, &c))
 	error(_("failure in as_cholmod_factor"));
     return ans;
 }
 
+
 /**
  * Copy the contents of f to an appropriate CHMfactor object and,
  * optionally, free f or free both f and its pointer to its contents.
@@ -947,6 +1109,18 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
     int *dims, *type;
     char *class = (char*) NULL;	/* -Wall */
 
+#define DOFREE_MAYBE					\
+    if(dofree) {					\
+	if (dofree > 0) cholmod_free_factor(&f, &c);	\
+	else /* dofree < 0 */ Free(f);			\
+    }
+
+    if(!chm_factor_ok(f)) {
+	DOFREE_MAYBE;
+	error(_("CHOLMOD factorization was unsuccessful"));
+	// error(_("previous CHOLMOD factorization was unsuccessful"));
+    }
+
     switch(f->xtype) {
     case CHOLMOD_REAL:
 	class = f->is_super ? "dCHMsuper" : "dCHMsimpl";
@@ -955,11 +1129,11 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
 	class = f->is_super ? "nCHMsuper" : "nCHMsimpl";
 	break;
     default:
+	DOFREE_MAYBE;
 	error(_("f->xtype of %d not recognized"), f->xtype);
     }
+
     ans = PROTECT(NEW_OBJECT(MAKE_CLASS(class)));
-    if (f->minor < f->n)
-	error(_("CHOLMOD factorization was unsuccessful"));
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = dims[1] = f->n;
 				/* copy component of known length */
@@ -997,13 +1171,11 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
 	       (int*)f->prev, f->n + 2);
 
     }
-    if(dofree) {
-	if (dofree > 0) cholmod_l_free_factor(&f, &c);
-	else /* dofree < 0 */ Free(f);
-    }
+    DOFREE_MAYBE;
     UNPROTECT(1);
     return ans;
 }
+#undef DOFREE_MAYBE
 
 /**
  * Drop the (unit) diagonal entries from a cholmod_sparse matrix
@@ -1018,7 +1190,7 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
  */
 void chm_diagN2U(CHM_SP chx, int uploT, Rboolean do_realloc)
 {
-    int i, n = chx->nrow, nnz = (int)cholmod_l_nnz(chx, &c),
+    int i, n = chx->nrow, nnz = (int)cholmod_nnz(chx, &c),
 	n_nnz = nnz - n, /* the new nnz : we will have removed  n entries */
 	i_to = 0, i_from = 0;
 
@@ -1026,7 +1198,7 @@ void chm_diagN2U(CHM_SP chx, int uploT, Rboolean do_realloc)
 	error(_("chm_diagN2U(<non-square matrix>): nrow=%d, ncol=%d"),
 	      n, chx->ncol);
 
-    if (!chx->sorted || !chx->packed) cholmod_l_sort(chx, &c);
+    if (!chx->sorted || !chx->packed) cholmod_sort(chx, &c);
 				/* dimensions and nzmax */
 
 #define _i(I) (   (int*) chx->i)[I]
@@ -1078,7 +1250,7 @@ void chm_diagN2U(CHM_SP chx, int uploT, Rboolean do_realloc)
 #undef _p
 
     if(do_realloc) /* shorten (i- and x-slots from nnz to n_nnz */
-	cholmod_l_reallocate_sparse(n_nnz, chx, &c);
+	cholmod_reallocate_sparse(n_nnz, chx, &c);
     return;
 }
 
